#!/bin/sh
# Этот скрипт настраивает следующие параметры модема
# AUTODIAL=0 Параметр автодозвона
# usbnetmode set to ECM  параметр указывающий интерфейс модема (RNDIS или TTYECM)
# Устанавливает IMEI на sim1

approvedModemFirmware="/run/ppp/peers/ApprovedModemFirmware.lst"
autodialDisable="/run/ppp/peers/autodialDisable"
exitCodeOK=0
exitCodeToReset=3
exitCodeToRestart=1
modemDevice="/dev/ttyUSB2"
switchToECM="/run/ppp/peers/switchToECM"
switchToSim0="/run/ppp/peers/switchToSim0"
timToFindProvider=15
timeToSwitchUsbMode=10
timeoutFromSimEI=5

# Процедура выполнения команд  на устройстве модем
# Входные параметры:
# $1 - строка с AT командой для модема
# $2 - шаблон ожидаемого ответа от модема
# $3 - максимальное время ожидания ответа модема (необязательная, по умолчанию равно 0,5 секунды)
# $4 - имя переменной в которую помещается вывод socat (необязательная)
# В случае нахождения шаблона в ответе возвращает exitCodeOK
# иначе возвращает exitCodeToReset

sendCommandToModem(){

    local command=$1
    local pattern=$2
    local timeout=$3
    local returnValue=$4

    if [ -z "$timeout" ]; then

        timeout="0.5"

    fi

    local commandResult=$(echo -e "$command" | socat -t $timeout -lf /tmp/socat.log - $modemDevice,crnl,b115200,raw,echo=0)
    echo "$commandResult" | sed '/^$/d'
    echo "$commandResult" | awk -v pattern=$pattern '{if ($0 ~ pattern) {exit 1} }'

    if [ $? -eq 0 ]; then

        echo -e "Failed to find pattern=$pattern in command=$command\n  commandResult=$commandResult"
        return $exitCodeToReset

    fi

    if [ -n "$returnValue" ]; then

        commandResult=$(echo "$commandResult" | awk -v pattern=$pattern '{if ($0 ~ pattern) {print $0; exit 1} }')
        eval $returnValue="'$commandResult'"

    fi

    return $exitCodeOK
}

# Проверяем версию прошивки
# Возвращает exitCodeOK в случае успеха, иначе exitCodeToReset

checkFirmware(){

    local firmware=''
    local pattern="+CGMR:"

    echo "Check firmware version ..."
    sendCommandToModem "AT+CGMR" $pattern 0.5 firmware

    # В найденной строке берем второе слово
    firmware=$(echo "$firmware" | awk -v pattern=$pattern '{if ($0 ~ pattern) {print $2; exit 1} }')

    if [ $? -ne 1 ]; then

        echo "Failed to get firmware"
        return $exitCodeToReset

    fi

    grep -q "^$firmware$" $approvedModemFirmware

    if [ $? -ne 0 ]; then

        echo "The modem has the wrong firmware $firmware"
        return $exitCodeToReset

    fi

    echo "Found modem firmware $firmware"

    return $exitCodeOK
}

# Проверяет IMEI на второй сим-карте
# Возвращает exitCodeOK в случае если IMEI установлен, иначе exitCodeToReset

checkIMEI(){

    local IMEI=''
    local pattern="+SIMEI:"

    echo "Check IMEI for sim1 ..."
    sendCommandToModem "AT+BINDSIM=1" "OK"
    sendCommandToModem "AT+SIMEI?" $pattern 0.5 IMEI

    if [ $? -ne 0 ]; then

        echo "Failed to get IMEI!"
        return $exitCodeToReset

    fi

    # В найденной строке ищем IMEI из 15 цифр
    echo "$IMEI" | grep -Eo "[0-9]{15}"

    return $?
}

# Прописываем IMEI на второй сим-карте
# Возвращает exitCodeOK в случае успеха, иначе exitCodeToReset

setIMEI(){

    local IMEI=''
    local pattern="+SIMEI:"

    echo "Check IMEI for sim0 ..."
    sendCommandToModem "AT+BINDSIM=0" "OK"
    sendCommandToModem "AT+SIMEI?" $pattern 0.5 IMEI

    # В найденной строке берем второе слово
    IMEI=$(echo "$IMEI" | awk -v pattern=$pattern '{if ($0 ~ pattern) {print $2; exit 1} }')

    if [ $? -ne 1 ]; then

        echo "Failed to get IMEI"
        return 1

    fi

    echo "Set IMEI($IMEI) for sim1"

    sendCommandToModem "AT+BINDSIM=1" "OK"
    sendCommandToModem "AT+SIMEI=$IMEI" "OK" $timeoutFromSimEI

    return $?
}

echo "++++ Start $0 ++++"

if [ ! -f $autodialDisable ]; then

    echo "File $autodialDisable not found"
    exit $exitCodeToReset

fi

if [ ! -f $switchToECM ]; then

    echo "File $switchToECM not found"
    exit $exitCodeToReset

fi

if [ ! -c $modemDevice ]; then

    echo "File $modemDevice not found"


    if [ -f $modemDevice ]; then

        rm $modemDevice

    fi
    exit $exitCodeToReset

fi

# Готовим дескриптор для chat
exec 3<>$modemDevice

# Настраиваем AUTODIAL
chat -v -f $autodialDisable >&3 <&3

case "$?" in

    0) # Успешно выключили autodial

        echo "Autodial is disabling"

    ;;
    4) # 4 - Завершился на первой строчке ABORT

        echo "Autodial was disabled"

    ;;
    *)

        echo "Failed to set Autodial to disable with code $temp"
        exit $exitCodeToReset

    ;;
esac

# Настраиваем usbnetmode
chat -v -f $switchToECM >&3 <&3

case "$?" in

    0) # Успешно  переключились в TTYECM

        echo "Modem switch to ECM mode"
        # задержка для ожидания перезагрузки модема
        sleep $timeToSwitchUsbMode

    ;;
    4) # 4 - Завершился на первой строчке ABORT

        echo "Modem was switched to ECM mode"

    ;;
    5) # 5 - Завершился на второй строчке ABORT

        echo "Modem was switched to ECM1 mode"

    ;;
    *)

        echo "Failed  to switch modem ECM mode with code $temp"
        exit $exitCodeToReset

    ;;
esac

checkFirmware

if [ $? -ne 0 ]; then

    echo "Failed to find right firmware"
    exit $exitCodeToReset

fi

checkIMEI

if [ $? -eq 0 ]; then

    echo "IMEI on sim1 was set"
    exit $exitCodeOK

fi

setIMEI

if [ $? -eq 0 ]; then
    echo "Set IMEI successful"
fi

exit $exitCodeOK